import { CommonModule } from "@angular/common";
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, HostListener, OnInit } from "@angular/core";
import { JobService } from "../../data-access/services/JobService.service";
import { NzTableModule } from 'ng-zorro-antd/table';
import { catchError, finalize, of, tap } from "rxjs";
import { JobModel } from "../../data-access/model/jobModel.model";
import { NzDividerModule } from 'ng-zorro-antd/divider';
import { NzButtonModule } from "ng-zorro-antd/button";
import { NzSpaceModule } from 'ng-zorro-antd/space';
import { NzModalModule, NzModalService } from 'ng-zorro-antd/modal';
import {FormsModule, ReactiveFormsModule } from "@angular/forms";
import { NzFormModule } from "ng-zorro-antd/form";
import { NzSelectModule } from 'ng-zorro-antd/select';
import { NzMessageService } from 'ng-zorro-antd/message';
import { NzAlertModule } from "ng-zorro-antd/alert";
import { NzNotificationService } from 'ng-zorro-antd/notification';
import { ViewJobComponent } from "./Action/JobDetail/JobDetail.component";
import { ModalJobComponent } from "./Action/ModalJob/Modal.component";

@Component({
  selector: "meu-admin-job",
  templateUrl: "./Job.component.html",
  standalone: true,
  imports: [
    CommonModule,
    NzTableModule,
    NzDividerModule,
    NzButtonModule,
    NzSpaceModule,
    NzModalModule,
    NzFormModule,
    ReactiveFormsModule,
    NzSelectModule,
    FormsModule,
    NzAlertModule,
    ViewJobComponent,
    ModalJobComponent
  ],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class JobComponent implements OnInit {
  isMobileView: boolean = false;
  searchTerm: string = '';

  //#region Variable
  selectedJob?: JobModel.Job;
  listOfJob: JobModel.Job[] = [];

  showModal: boolean = false;
  isEdit : boolean = false;
  showViewModalComponent: boolean = false;

  // Công việc hiện tại để chỉnh sửa hoặc xem chi tiết
  currentEditJob: JobModel.Job | null = null;
  currentViewJob: JobModel.Job | null = null;
  //#endregion

  //#region LoadData
  constructor(
    private _cdr: ChangeDetectorRef,
    private jobService: JobService,
    private modal: NzModalService,
    private message: NzMessageService,
    private notification: NzNotificationService
  ) { }

  loadDataFromServer(): void {
    this.jobService.jobsGet().pipe(
      tap((res) => {
        if (res.responseData) {
          this.listOfJob = res.responseData.rows;
        }
        this._cdr.detectChanges();
      }),
      catchError((err) => {
        this.listOfJob = [];
        this.notification.error('Lỗi', 'Không tải được dữ liệu công việc.');
        return of(null);
      })
    ).subscribe();
  }

  ngOnInit(): void {
    this.loadDataFromServer();
    this.checkScreenSize(); // Kiểm tra kích thước màn hình ban đầu
  }

  onCloseViewModal(): void {
    this.showViewModalComponent = false;
  }
  onSubmit(): void {
    this.isEdit = false;
    this.showModal = false;
    this._cdr.detectChanges();
    this.loadDataFromServer();
  }
  onCloseModal(): void {
    this.showModal = false;
  }
  //#endregion

  //#region Handlers

  // Handler xem chi tiết công việc
  handleViewJob(id: string): void {
    this.jobService.getJobById(id)
      .pipe(
        tap((res) => {
            this.showViewModalComponent = true;
            this.currentViewJob = res.responseData as JobModel.Job;
            this._cdr.detectChanges();
        }),
        catchError((err) => {
          this.message.error(err.message || 'An error occurred while fetching job data.', { nzDuration: 2500 });
          return of(null);
        })
      )
      .subscribe();
  }

  // Handler xóa công việc
  onDelete(id: string): void {
    this.jobService.deleteJob(id)
      .pipe(
        tap((res) => {
          this.message.success(res.message, { nzDuration: 2500 });
          this._cdr.markForCheck();
          this.loadDataFromServer();
        }),
        catchError((err) => {
          this.message.error(err.message, { nzDuration: 2500 });
          return of(null);
        })
      )
      .subscribe();
  }

  // Xử lý khi nhấn nút "Thêm công việc"
  addJob(): void {
    this.showModal = true;
  }

  // Xử lý khi nhấn nút "Sửa"
  onEdit(data: JobModel.Job): void {
    this.isEdit = true;
    this.currentEditJob = data;
    this.showModal= true;
  }

  // Xử lý khi nhấn nút "Xem chi tiết"
  JobDetail(id: string): void {
    this.handleViewJob(id);
  }

  // Hiển thị hộp thoại xác nhận xóa
  showDeleteConfirm(id: string): void {
    this.modal.confirm({
      nzTitle: 'Bạn có chắc rằng xóa công việc này không?',
      nzOkText: 'Xóa',
      nzOkType: 'primary',
      nzOkDanger: true,
      nzOnOk: () => this.onDelete(id),
      nzCancelText: 'Trở về',
    });
  }

  //#endregion

  //#region HostListener
  @HostListener('window:resize', ['$event'])
  onResize(event: any) {
    this.checkScreenSize();
  }

  checkScreenSize() {
    this.isMobileView = window.innerWidth < 768;
    this._cdr.detectChanges();
  }

  isMobile(): boolean {
    return this.isMobileView;
  }
  //#endregion
}
